@@ -55,15 +55,9 @@
 #'
 #' @export
 #'
-send_aio <- function(con,
-                     data,
-                     mode = c("serial", "raw"),
-                     timeout = NULL) {
-
+send_aio <- function(con, data, mode = c("serial", "raw"), timeout = NULL)
   data <- .Call(rnng_send_aio, con, data, mode, timeout, environment())
 
-}
-
 #' Receive Async
 #'
 #' Receive data asynchronously over a connection (Socket, Context or Stream).
@@ -125,12 +119,9 @@ recv_aio <- function(con,
                               "integer", "logical", "numeric", "raw"),
                      timeout = NULL,
                      keep.raw = FALSE,
-                     n = 65536L) {
-
+                     n = 65536L)
   result <- .Call(rnng_recv_aio, con, mode, timeout, keep.raw, n, environment())
 
-}
-
 # Core aio functions -----------------------------------------------------------
 
 #' Call the Value of an Asynchronous AIO Operation
