@@ -54,14 +54,9 @@ messenger <- function(url, auth = NULL) {
   key <- c(comb, as.integer(lock)[comb])
 
   sock <- .Call(rnng_messenger, url)
-  is.integer(sock) && return(invisible(sock))
-
-  nano_init(warn = "none")
   on.exit(expr = {
     send(sock, data = writeBin(":d ", raw()), mode = 2L, block = FALSE)
     .Call(rnng_close, sock)
-    options(warn = getOption("nanonext.original.warn"), nanonext.original.warn = NULL)
-    invisible()
   })
   cat("\n", file = stdout())
   intro <- unlist(strsplit("nanonext messenger", ""))
@@ -104,5 +99,7 @@ messenger <- function(url, auth = NULL) {
     }
   }
 
+  invisible()
+
 }
 
