@@ -21,11 +21,16 @@
 #' nano cURL - a minimalist http(s) client.
 #'
 #' @param url the URL address.
-#' @param async [default FALSE] logical value whether to perform actions async.
+#' @param async [default FALSE] logical value whether to perform an async request,
+#'     in which case an 'ncurlAio' is returned instead of a list.
 #' @param convert [default TRUE] logical value whether to attempt conversion of
-#'     the received raw bytes to a character vector.
+#'     the received raw bytes to a character vector. Supplying a non-logical
+#'     value will error.
 #' @param follow [default FALSE] logical value whether to automatically follow
-#'     redirects. See 'redirects' section below.
+#'     redirects (ignored for async requests). If FALSE, or for async requests,
+#'     the redirect address is returned as a character string at \code{$data}
+#'     and the HTTP status code will be within the 300 range. Supplying a
+#'     non-logical value will error.
 #' @param method (optional) the HTTP method (defaults to 'GET' if not specified).
 #' @param headers (optional) a named list or character vector specifying the
 #'     HTTP request headers e.g. \code{list(`Content-Type` = "text/plain")} or
@@ -41,7 +46,8 @@
 #'
 #' @return Named list of 4 elements:
 #'     \itemize{
-#'     \item{\code{$status}} {- integer HTTP repsonse status code (200 - OK).}
+#'     \item{\code{$status}} {- integer HTTP repsonse status code (200 - OK).
+#'     Use \code{\link{status_code}} for a translation of the meaning.}
 #'     \item{\code{$headers}} {- named list of response headers supplied in
 #'     'response' or NULL if unspecified.}
 #'     \item{\code{$raw}} {- raw vector of the received resource (use
@@ -54,14 +60,6 @@
 #'     Or else, if \code{async = TRUE}, an 'ncurlAio' (object of class 'ncurlAio'
 #'     and 'recvAio') (invisibly).
 #'
-#' @section Redirects:
-#'
-#'     If redirects are not followed (the default), the redirect address is
-#'     returned as a character string.
-#'
-#'     For async requests, the redirect address is returned as a character
-#'     string at \code{$raw} and \code{$data} will be NULL.
-#'
 #' @examples
 #' ncurl("https://httpbin.org/get", response = c("date", "server"))
 #' ncurl("http://httpbin.org/put",,,,"PUT", list(Authorization = "Bearer APIKEY"), "hello world")
@@ -77,20 +75,8 @@ ncurl <- function(url,
                   headers = NULL,
                   data = NULL,
                   response = NULL,
-                  pem = NULL) {
-
-  if (async) {
-
-    response
-    context <- .Call(rnng_ncurl_aio, url, convert, method, headers, data, pem, environment())
-
-  } else {
-
-    res <- .Call(rnng_ncurl, url, convert, method, headers, data, response, pem)
-    is.character(res) && follow && return(eval(`[[<-`(match.call(), 2L, res)))
-    res
-
-  }
-
-}
+                  pem = NULL)
+  if (async)
+    data <- .Call(rnng_ncurl_aio, url, convert, method, headers, data, pem, environment()) else
+      .Call(rnng_ncurl, url, convert, follow, method, headers, data, response, pem)
 
